@@ -1,7 +1,8 @@
 #!perl
 
 use Test::More;
-use strict; use warnings;
+use strict;
+use warnings;
 our $es;
 my $r;
 
@@ -16,4 +17,4 @@ note "Current server is "
     . $r->{number};
 note "Built: " . $r->{date};
 
-1;
\ No newline at end of file
+1;
